#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <assert.h>
#include <strings.h>

typedef struct {
    int bidreqId, decNum, dec, overallDec, sumRep, numReps, 
	buyerAuctions, buyerCancels, numbids, type, sellRep;
    double avgbid, cost, bidAmt;
    double stype;
    double stype2;
    int btype;
} Bid;

#define MAX_LINE_SIZE 10000
char line[MAX_LINE_SIZE];
int lineno = 0;

Bid bids[200000];
int numb = 0;

int auctionBids[10000];
int numAuctions = 0;
int prevBidReq = -1;

char *ReadLine(FILE *fp)
{
    char *res;

    lineno++;
    res = fgets(line, MAX_LINE_SIZE, fp);
    if (!res)
	return res;

    Bid *b = &bids[numb];
    
    sscanf(line, "%d %d %d %d %d %d %d %d %d %d %lf %lf %d %lf",
	   &b->bidreqId, &b->decNum,
	   &b->dec, &b->overallDec, &b->sumRep, &b->numReps,
	   &b->buyerAuctions, &b->buyerCancels, &b->numbids,
	   &b->type, &b->avgbid, &b->cost, &b->sellRep, &b->bidAmt);

    if (b->bidreqId != prevBidReq) {
	auctionBids[numAuctions] = numb;
	numAuctions++;
    }
    prevBidReq = b->bidreqId;
    numb++;

    return res;
}

typedef struct {
    double cost;
    double weight;
    double cumwt;
} CostType;

CostType cost1[50000], cost2[50000], cost3[50000];
int numCosts = 0;

#define NUMITERS 100
double percentiles[3][100][NUMITERS];
int iternum = 0;

int comparevals(const void *s1, const void *s2)
{
    double c1 = ((const CostType *)s1)->cost;
    double c2 = ((const CostType *)s2)->cost;

    if (c1 > c2)
	return 1;
    else if (c1 == c2)
	return 0;
    else
	return -1;
}

int comparedoubles(const void *s1, const void *s2)
{
    double c1 = *((const double *)s1);
    double c2 = *((const double *)s2);

    if (c1 > c2)
	return 1;
    else if (c1 == c2)
	return 0;
    else
	return -1;
}

void ProcessCosts(char *fname)
{
    FILE *fp = fopen(fname, "r");

    char *res = fgets(line, MAX_LINE_SIZE, fp);
    Bid tmp;
    Bid *b = &tmp;
    numCosts = 0;
    while (res) {
	double c1, c2, c3;
	double v1, v2, v3;
	sscanf(line, "%d %d %d %d %d %d %d %d %d %d %lf %lf %d %lf %lf %lf %lf %lf %lf %lf %lf %lf",
	       &b->bidreqId, &b->decNum,
	       &b->dec, &b->overallDec, &b->sumRep, &b->numReps,
	       &b->buyerAuctions, &b->buyerCancels, &b->numbids,
	       &b->type, &b->avgbid, &b->cost, &b->sellRep, &b->bidAmt,
	       &b->stype, &b->stype2, &c1, &v1, &c2, &v2, &c3, &v3);
	       
	cost1[numCosts].cost = c1;
	cost1[numCosts].weight = 1 - b->stype - b->stype2;
	cost2[numCosts].cost = c2;
	cost2[numCosts].weight = b->stype2;
	cost3[numCosts].cost = c3;
	cost3[numCosts].weight = b->stype;
	numCosts++;

	res = fgets(line, MAX_LINE_SIZE, fp);
    }
    qsort(cost1, numCosts, sizeof(CostType), comparevals);
    assert(cost1[0].cost < cost1[1].cost);
    qsort(cost2, numCosts, sizeof(CostType), comparevals);
    assert(cost2[0].cost < cost2[1].cost);
    qsort(cost3, numCosts, sizeof(CostType), comparevals);
    assert(cost3[0].cost < cost3[1].cost);

    double cumwt, tcumwt;
    int nextval;

    cumwt = 0;
    for (int i=0; i<numCosts; i++)
	cumwt += cost1[i].weight;

    tcumwt = 0;
    nextval = 1;
    for (int i=0; i<numCosts; i++) {
	tcumwt += cost1[i].weight;	
	cost1[i].cumwt = tcumwt/cumwt;
	if (100*cost1[i].cumwt > nextval) {
	    percentiles[0][nextval][iternum] = cost1[i].cost;
	    nextval++;
	}
    }

    cumwt = 0;
    for (int i=0; i<numCosts; i++)
	cumwt += cost2[i].weight;

    tcumwt = 0;
    nextval = 1;
    for (int i=0; i<numCosts; i++) {
	tcumwt += cost2[i].weight;	
	cost2[i].cumwt = tcumwt/cumwt;
	if (100*cost2[i].cumwt > nextval) {
	    percentiles[1][nextval][iternum] = cost2[i].cost;
	    nextval++;
	}
    }

    cumwt = 0;
    for (int i=0; i<numCosts; i++)
	cumwt += cost3[i].weight;

    tcumwt = 0;
    nextval = 1;
    for (int i=0; i<numCosts; i++) {
	tcumwt += cost3[i].weight;	
	cost3[i].cumwt = tcumwt/cumwt;
	if (100*cost3[i].cumwt > nextval) {
	    percentiles[2][nextval][iternum] = cost3[i].cost;
	    nextval++;
	}
    }
}

int
main(int argc, char **argv)
{
    FILE *fp = fopen(argv[1], "r");

    char *res = ReadLine(fp);
    while (res) {
	res = ReadLine(fp);
    }
    printf("%d %d\n", numb, numAuctions);
    auctionBids[numAuctions] = numb;

    for (int i=0; i<NUMITERS; i++) {
	FILE *fpout = fopen("sampled-aucs.txt", "w");
	for (int j=0; j<numAuctions; j++) {
	    int r = random() % numAuctions;
	    for (int k=auctionBids[r]; k<auctionBids[r+1]; k++) {
		Bid *b = &bids[k];
		fprintf(fpout, "%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%lf\t%lf\t%d\t%lf\n",
			b->bidreqId, b->decNum, b->dec, b->overallDec,
			b->sumRep, b->numReps, b->buyerAuctions, b->buyerCancels,
			b->numbids, b->type, b->avgbid, b->cost, b->sellRep,
			b->bidAmt);
	    }
	}
	fclose(fpout);
	system("./sortkde3 sampled-aucs.txt synthdata-weights2.txt");
	system("/Applications/Stata/StataMP.app/Contents/MacOS/stata-mp -b synthdata.do");
	char command[200];
	sprintf(command, "./simulate3-clean synthdata-weights2.txt synthdata-costs.%d.txt", i);
	system(command);
	
	char fname[30];
	sprintf(fname, "synthdata-costs.%d.txt", i);
	ProcessCosts(fname);
	iternum++;
    }
    FILE *cfp1, *cfp2, *cfp3;
    cfp1 = fopen("costs-dist1.txt", "w");
    cfp2 = fopen("costs-dist2.txt", "w");
    cfp3 = fopen("costs-dist3.txt", "w");
    for (int i=0; i<iternum; i++) {
	for (int j=1; j<=99; j++)
	    fprintf(cfp1, "%lf ", percentiles[0][j][i]);
	fprintf(cfp1, "\n");
	for (int j=1; j<=99; j++)
	    fprintf(cfp2, "%lf ", percentiles[1][j][i]);
	fprintf(cfp2, "\n");
	for (int j=1; j<=99; j++)
	    fprintf(cfp3, "%lf ", percentiles[2][j][i]);
	fprintf(cfp3, "\n");
    }

    for (int j=1; j<=99; j++) {
	qsort(&percentiles[0][j][0], iternum, sizeof(double), comparedoubles);
	qsort(&percentiles[1][j][0], iternum, sizeof(double), comparedoubles);
	qsort(&percentiles[2][j][0], iternum, sizeof(double), comparedoubles);

	double sum, sumsq;
	sum = 0; sumsq = 0;
	for (int i=0; i<iternum; i++) {
	    sum += percentiles[0][j][i];
	    sumsq += percentiles[0][j][i]*percentiles[0][j][i];
	}
	fprintf(cfp1, "%lf %lf %lf %lf %lf\n", sum/iternum, percentiles[0][j][5],
		percentiles[0][j][95], percentiles[0][j][50],
		sqrt(sumsq/iternum - (sum/iternum)*(sum/iternum)));

	sum = 0; sumsq = 0;
	for (int i=0; i<iternum; i++) {
	    sum += percentiles[1][j][i];
	    sumsq += percentiles[1][j][i]*percentiles[1][j][i];
	}
	fprintf(cfp2, "%lf %lf %lf %lf %lf\n", sum/iternum, percentiles[1][j][5],
		percentiles[1][j][95], percentiles[1][j][50],
		sqrt(sumsq/iternum - (sum/iternum)*(sum/iternum)));

	sum = 0; sumsq = 0;
	for (int i=0; i<iternum; i++) {
	    sum += percentiles[2][j][i];
	    sumsq += percentiles[2][j][i]*percentiles[2][j][i];
	}
	fprintf(cfp3, "%lf %lf %lf %lf %lf\n", sum/iternum, percentiles[2][j][5],
		percentiles[2][j][95], percentiles[2][j][50],
		sqrt(sumsq/iternum - (sum/iternum)*(sum/iternum)));
    }	
    fclose(cfp1);
    fclose(cfp2);
    fclose(cfp3);
}
