#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <assert.h>
#include <strings.h>

/*
input schema:
BidRequestId
DecisionNo
Decision
OverallDecision
SumRep
NumReps
PreviousBids
PreviousCancels
numbids
ttype
avbid
cost
sellRep
BidAmount
aucwt
aucwt2

*/

int bidreqId, decNum, dec, overallDec, sumRep, numReps, buyerAuctions, buyerCancels, numbids, type, sellrep;
double avgBid, cost, bidAmt, aucWt, aucWt2;

typedef struct {
    double bidAmt;
    int sellrep;
} BidType;

#define MAXBIDS 1000
BidType bids[MAXBIDS];

#define MAX_LINE_SIZE 10000
char line[MAX_LINE_SIZE];

int lineno = 0;

char *ReadLine(FILE *fp)
{
    char *res;

    lineno++;
    res = fgets(line, MAX_LINE_SIZE, fp);
    if (!res)
	return res;
    
    sscanf(line, "%d %d %d %d %d %d %d %d %d %d %lf %lf %d %lf %lf %lf",
	   &bidreqId, &decNum, &dec, &overallDec, &sumRep, &numReps, 
	   &buyerAuctions, &buyerCancels, &numbids, &type,
	   &avgBid, &cost, &sellrep, &bidAmt, &aucWt, &aucWt2);
    return res;
}

char *ReadBid(FILE *fp)
{
    char *res = ReadLine(fp);

    bids[0].sellrep = sellrep;
    bids[0].bidAmt = bidAmt;
    return res;
}

#define NUMBTYPES	3
double cdfs[NUMBTYPES][3][600];
double icdfs[NUMBTYPES][3][1000];

void ReadCdfs()
{
    for (int i=0; i<NUMBTYPES; i++) {
	for (int s=0; s<3; s++) {
	    char fname[100];
	    sprintf(fname, "cdf.out.%d.%d", i, s);
	    FILE *fp = fopen(fname, "r");
	    char *res = fgets(line, 100, fp);
	    int lineno = 0;
	    int nextval = 0;
	    while (res) {
		sscanf(line, "%lf", &cdfs[i][s][lineno]);
		while (cdfs[i][s][lineno] > nextval/1000.0) {
		    icdfs[i][s][nextval] = lineno;
		    nextval++;
		}
		lineno++;
		res = fgets(line, 100, fp);
	    }
	    for (;nextval<1000; nextval++)
		icdfs[i][s][nextval] = 600;
	}
    }
}


void GetNextBid(int numbids, double buyrep, int numBuyReps, int *sellrep)
{
    int r = random() % 3;
    *sellrep = 8 + r;
}

double c6_price = -0.01;
double c6_sellrep = 2;
			           
double c7_lnnumreps = 0.5;
double c7_buyrep = 0.1;
double c7_lnprevcancel = -1;
double c7_cons = -1;
double nestCorr = 0.5;

/*
        decisionno:bidamount    decisionno:nsellrep     type2:lnnumreps type2:buyrep    type2:lnprevcancel      type2:_cons     type1_tau:_cons type2_tau:_cons 
y1      -.01025389       1.9961843       .42920992       .10628854      -.89694821      -.99933217               1       .49168695  
 */
void ReadParams()
{
    FILE *fp = fopen("coeff.txt", "r");
    char *res = fgets(line, MAX_LINE_SIZE, fp);
    res = fgets(line, MAX_LINE_SIZE, fp);
    char ign1[20];
    double ign2;
    sscanf(line, "%s %lf %lf %lf %lf %lf %lf %lf %lf",
	   ign1, &c6_price, &c6_sellrep, &c7_lnnumreps,
	   &c7_buyrep, &c7_lnprevcancel, &c7_cons, &ign2, &nestCorr);
}


double GetBidAmt(int sellrep, int type)
{
    int nt, srt;

    if (sellrep == 8)
	srt = 0;
    else if (sellrep == 9)
	srt = 1;
    else
	srt = 2;

    int btype = srt;
    int r = random() % 1000;
    switch (type) {
    case 1: return icdfs[btype][0][r];
    case 2: return icdfs[btype][1][r];
    default: return icdfs[btype][2][r];
    }
    
}

void SimulateOnce(double *rval1, double *rval2, int type)
{
    *rval1 = 0;
    *rval2 = 0;
    
    double lnprevcancel = log(buyerCancels + 1);
    double lnnumreps = log(numReps + 1);
    double lnsucreqs = log(buyerAuctions - buyerCancels + 1);
    double buyrep = (numReps > 0 ? ((double)sumRep/numReps) : 0);

    // Generate new bids to be considered with this one
    for (int i=1; i<numbids; i++) {
	double bid, ssumrep, snumreps;
	GetNextBid(numbids, sumRep, numReps, &bids[i].sellrep);

	bids[i].bidAmt = GetBidAmt(bids[i].sellrep, type);
	if (bids[i].bidAmt < 0) bids[i].bidAmt = 10;
    }

    double maxNestUtil = 0;
    double nestUtils[1000];

    double total = 0;
    for (int i=0; i<numbids; i++)
	total += bids[i].bidAmt;
    
    for (int i=0; i<numbids; i++) {

	double bid = bids[i].bidAmt;
	double sellrep = bids[i].sellrep;

	double util = 0;

	util += (c6_price*bid + c6_sellrep*(sellrep-8));
	util = util/nestCorr;
	    
	nestUtils[i] = util;
	if (!isfinite(nestUtils[i])) {
	    printf("error\n");
	}

	if (maxNestUtil < util || i == 0)
	    maxNestUtil = util;
    }

    // get the math precision right
    double bidNestUtil = 0;
    for (int i=0; i<numbids; i++) {
	bidNestUtil += exp(nestUtils[i] - maxNestUtil);
    }
    double incVal = log(bidNestUtil) + maxNestUtil;

    // now that we have inclusive value, calculate probability of
    // bid 0 being picked and the derivative of that probability
    {
	double topUtil = c7_cons;


	topUtil +=  (       c7_lnnumreps*lnnumreps
		     +         c7_buyrep*buyrep
		     +   c7_lnprevcancel*lnprevcancel);

	double A = exp(topUtil + nestCorr*incVal);
	double B = exp(nestUtils[0] - maxNestUtil);
	double C = bidNestUtil;

	if (!isfinite(A) || !isfinite(B) || !isfinite(C)) {
	    printf("error\n");
	}
	
	double prob = A/(1+A) * B/C;
	double probDer = (  prob * c6_price
			    * (B/(C*(1+A)) + 1/nestCorr*(1 - B/C)));

	*rval1 = prob;
	*rval2 = probDer;
    }
}


int doneFlag = 0;

int
main(int argc, char **argv)
{
    FILE *fp = fopen(argv[1], "r");
    FILE *fpout = fopen(argv[2], "w");

    ReadCdfs();
    ReadParams();
    // ignore the first line
    char *res = ReadLine(fp);
    res = ReadLine(fp);
    while (res) {
	for (int i=0; i<numbids; i++) {
	    ReadBid(fp);
	    if (bidAmt >= 5000)
		continue;

	    line[strlen(line)-1] = '\0';
	    double cost_t[3];
	    double val1_t[3];
	    double val2_t[3];
	    for (int type=1; type<=3; type++) {
		double val1 = 0, val2 = 0, val3 = 0, val4 = 0;
#define NUMITERS 1000
		for (int k=0; k<NUMITERS; k++) {
		    double t1, t2;
		    SimulateOnce(&t1, &t2, type);
		    val1 += t1;
		    val2 += t2;
		}
		double cost = 0.85*(bids[0].bidAmt
				    + (val1/NUMITERS)/(val2/NUMITERS));
		cost_t[type-1] = cost;
		val1_t[type-1] = val1/NUMITERS;
		val2_t[type-1] = val2/NUMITERS;
	    }
	    fprintf(fpout, "%s\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\n", line,
		    cost_t[0], val1_t[0]/val2_t[0],
		    cost_t[1], val1_t[1]/val2_t[1],
		    cost_t[2], val1_t[2]/val2_t[2]);
	}
	res = ReadLine(fp);
    }
}

/*

cat /tmp/f | awk '{ print $17, 1-$15-$16; }' > /tmp/cdf1
cat /tmp/f | awk '{ print $19, $16; }' > /tmp/cdf2
cat /tmp/f | awk '{ print $21, $15; }' > /tmp/cdf3
  
cat /tmp/cdf1 | awk 'BEGIN { sum = 0; } { sum += $2; } END { print sum; }'
cat /tmp/cdf2 | awk 'BEGIN { sum = 0; } { sum += $2; } END { print sum; }'
cat /tmp/cdf3 | awk 'BEGIN { sum = 0; } { sum += $2; } END { print sum; }'

cat /tmp/cdf3 | sort -n | awk 'BEGIN { sum = 0; } { sum += $2; print $1, sum/13234.4; }' > /tmp/cdf3.gph
cat /tmp/cdf2 | sort -n | awk 'BEGIN { sum = 0; } { sum += $2; print $1, sum/13946.8; }' > /tmp/cdf2.gph
cat /tmp/cdf1 | sort -n | awk 'BEGIN { sum = 0; } { sum += $2; print $1, sum/12838.8; }' > /tmp/cdf1.gph

*/
