#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <assert.h>
#include <strings.h>

/*
input schema:
BidRequestId	
DecisionNo
Decision
OverallDecision
SumRep
NumReps
PreviousBids
PreviousCancels
numbids
ttype
avbid
cost
SellRep
BidAmount
*/

typedef struct {
    int bidreqId, decNum, dec, overallDec, sumRep, numReps, 
	buyerAuctions, buyerCancels, numbids, type, sellRep;
    double avgbid, cost, bidAmt;
    double stype;
    double stype2;
    int btype;
} Bid;

#define MAX_LINE_SIZE 10000
char line[MAX_LINE_SIZE];


double low_prob = 0.000001;

int lineno = 0;

char *ReadLine(FILE *fp, Bid *b)
{
    char *res;

    lineno++;
    res = fgets(line, MAX_LINE_SIZE, fp);
    if (!res)
	return res;
    
    sscanf(line, "%d %d %d %d %d %d %d %d %d %d %lf %lf %d %lf",
	   &b->bidreqId, &b->decNum,
	   &b->dec, &b->overallDec, &b->sumRep, &b->numReps,
	   &b->buyerAuctions, &b->buyerCancels, &b->numbids,
	   &b->type, &b->avgbid, &b->cost, &b->sellRep, &b->bidAmt);

    b->btype = (b->sellRep-8);
    b->stype = (b->avgbid > 300 ? 0.8 : 0.2);
    b->stype2 = (b->avgbid > 250 ? 0.65*(1-b->stype) : 0.35*(1-b->stype));
    assert(b->stype + b->stype2 <= 1.0);
    return res;
}

Bid bids[200000];
int numb = 0;

int numbtypes = 3;
#define MAXBTYPES 200
double kdePdf[MAXBTYPES][3][600];

void InitBids()
{
    for (int i=0; i<numb; ) {
	int startb = i;
	int endb = i+bids[i].numbids;
	double chosen = 0;
	for (int k=startb; k<endb; k++) {
	    Bid *b = &bids[k];
	    if (b->dec) {
		if (chosen < b->bidAmt)
		    chosen = b->bidAmt;
	    }
	}
	if (chosen > 200) {
	    for (int k=startb; k<endb; k++) {
		Bid *b = &bids[k];		
		b->stype = 0.5;
		b->stype2 = 0.5;
	    }
	}
	i = endb;
    }
}

void PrintEvalReadKde(int cdf)
{

    for (int i=0; i<numbtypes; i++) {

	for (int s=0; s<3; s++) {
	    FILE *fpp = fopen("prices.txt", "w");
	    FILE *fpw = fopen("weights.txt", "w");
	
	    for (int j=0; j<numb; j++) {
		Bid *b = &bids[j];
		if (b->btype == i) {
		    fprintf(fpp, "%lf\n", b->bidAmt);
		    fprintf(fpw, "%lf\n", (s == 0 ? 1 - b->stype - b->stype2
					   : (s == 1 ? b->stype2 : b->stype)));
		}
	    }
	    fclose(fpp);
	    fclose(fpw);
	    if (cdf) {
		system("/usr/bin/matlab -nodisplay -nosplash < cdfcmds.m > /dev/null");
		char cmd[100];
		sprintf(cmd, "/bin/cp cdf.out cdf.out.%d.%d", i, s);
		system(cmd);
		continue;
	    }
	    system("/usr/bin/matlab -nodisplay -nosplash < cmds.m > /dev/null");
	    FILE *fpk = fopen("kde.out", "r");
	    char line[100];
	    char *res = fgets(line, 100, fpk);
	    int lineno = 0;
	    while (res) {
		sscanf(line, "%lf", &kdePdf[i][s][lineno]);
		if (kdePdf[i][s][lineno] < low_prob)
		    kdePdf[i][s][lineno] = low_prob;
		lineno++;
		res = fgets(line, 100, fpk);  
	    }
	    fclose(fpk);
	}
    }
}

double UpdateS()
{
    double sum = 0, sum2 = 0;
    int numAuc = 0;
    for (int i=0; i<numb;) {
	sum += bids[i].stype;
	sum2 += bids[i].stype2;
	numAuc++;
	i += bids[i].numbids;
    }

    double avgS = sum/numAuc;
    double avgS2 = sum2/numAuc;
    double scumdiff = 0;

    printf("S: %lf %lf\n", avgS, avgS2);

    for (int i=0; i<numb; ) {
	int startb = i;
	int endb = i+bids[i].numbids;
	double f1 = 0, f2 = 0, f3 = 0, f4 = 0;
	
	for (int k=startb; k<endb; k++) {
	    int btype = bids[k].btype;
	    int amt = (int)bids[k].bidAmt;

	    f1 += log(kdePdf[btype][2][amt]);
	    f2 += log(kdePdf[btype][1][amt]);
	    f3 += log(kdePdf[btype][0][amt]);

	    assert(isfinite(f1) && isfinite(f2) && isfinite(f3) && isfinite(f4));
	}

	f1 += log(avgS);
	f2 += log(avgS2);
	f3 += log(1-avgS-avgS2);
	assert(isfinite(f1) && isfinite(f2) && isfinite(f3) && isfinite(f4));

	double newS, newS2;
	double oldS = bids[i].stype, oldS2 = bids[i].stype2;

	double fmax = f1;
	fmax = (fmax > f2 ? fmax : f2);
	fmax = (fmax > f3 ? fmax : f3);

	newS = exp(f1-fmax)/(exp(f1-fmax) + exp(f2-fmax) + exp(f3-fmax));
	newS2 = exp(f2-fmax)/(exp(f1-fmax) + exp(f2-fmax) + exp(f3-fmax));
	
	double diff = (newS > oldS ? newS - oldS : oldS - newS);
	assert(newS <= 1 && oldS <= 1 && diff <= 1);
	scumdiff += diff;
	
	diff = (newS2 > oldS2 ? newS2 - oldS2 : oldS2 - newS2);
	assert(newS2 <= 1 && oldS2 <= 1 && diff <= 1);
	scumdiff += diff;

	for (int k=startb; k<endb; k++) {
	    bids[k].stype = newS;
	    bids[k].stype2 = newS2;
	}

	i = endb;
    }
    return scumdiff/(2*numAuc);
}

int
main(int argc, char **argv)
{
    FILE *fp = fopen(argv[1], "r");

    char *res = ReadLine(fp, &bids[numb]);
    while (res) {
	int num = bids[numb].numbids;
	for (int i=0; i<num; i++) {
	    ReadLine(fp, &bids[numb]);
	    numb++;
	}
	res = ReadLine(fp, &bids[numb]);
    }
    printf("%d\n", numb);
    
    int iter = 0;
    while (1) {
	double diff = 0;
	PrintEvalReadKde(0);
	if ((diff = UpdateS()) < 0.000001)
	    break;
	printf("Iter: %d %f\n", iter++, diff);
    }

    FILE *fpo = fopen(argv[2], "w");
    fprintf(fpo, "BidRequestId\tDecisionNo\tDecision\tOverallDecision\tSumRep\tNumReps\tPreviousBids\tPreviousCancels\tnumbids\tttype\tavbid\tcost\tsellRep\tBidAmount\taucwt\taucwt2\n");
    for (int i=0; i<numb;) {
	int nb = bids[i].numbids;
	Bid *b = &bids[i];
	fprintf(fpo, "%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%lf\t%lf\t%d\t%lf\t%lf\t%lf\n",
		b->bidreqId, 1, (b->overallDec == 1 ? 1 : 0), b->overallDec,
		b->sumRep, b->numReps, b->buyerAuctions, b->buyerCancels,
		b->numbids, b->type, b->avgbid, 0.0, 0,	0.0, b->stype, b->stype2);
		
	for (int j=0; j<nb; j++, i++) {
	    Bid *b = &bids[i];
	    fprintf(fpo, "%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%lf\t%lf\t%d\t%lf\t%lf\t%lf\n",
		    b->bidreqId, b->decNum, b->dec, b->overallDec,
		    b->sumRep, b->numReps, b->buyerAuctions, b->buyerCancels,
		    b->numbids, b->type, b->avgbid, b->cost, b->sellRep,
		    b->bidAmt, b->stype, b->stype2);
	}
    }
    fclose(fpo);

    PrintEvalReadKde(1);    
}

